//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "VKTrackingAction.hh"
#include "VKRecorderBase.hh"
#include "VKTrajectory.hh"
#include "VKUserTrackInformation.hh"

#include "G4TrackingManager.hh"
#include "G4Track.hh"
#include "G4ParticleTypes.hh"


/*
 * Notes for self: 
 *
 *   VKTrackingAction inherits protected member 'fpTrackingManager' 
 *   from G4UserTrackingAction.
 *
 *   "&" is bitwise "AND" operator, e.g., 010 & 001 = 000  (2 & 1 = 0)
 *   "|" is bitwise "OR" operator,  e.g., 010 | 001 = 011  (2 | 1 = 3)
 *   "~" is bitwise "NOT" operator, e.g., ~010 = 101  (~2 = 5) 
 *   
 *   Also note that 0 corresponds to "false" while all integers >=1 
 *   correspond to "true".
 */


VKTrackingAction::VKTrackingAction(VKRecorderBase* r)
  : fRecorder(r) 
{
    // do nothing
}


void VKTrackingAction::PreUserTrackingAction(const G4Track* aTrack)
{
  // Use custom trajectory class
  fpTrackingManager -> SetTrajectory(new VKTrajectory(aTrack));

  // This user track information is only relevant to the photons
  fpTrackingManager -> SetUserTrackInformation(new VKUserTrackInformation);
}


void VKTrackingAction::PostUserTrackingAction(const G4Track* aTrack)
{
    // Trajectory
    VKTrajectory* trajectory = 
        (VKTrajectory*) (fpTrackingManager->GimmeTrajectory());
  
    // Track information
    VKUserTrackInformation* trackInformation = 
        (VKUserTrackInformation*) (aTrack->GetUserInformation());

    // Particle definition
    G4ParticleDefinition* particle = aTrack->GetDefinition();

    // Consider photons separately
    if (particle == G4OpticalPhoton::OpticalPhotonDefinition())
    {
        //
        // ??? NOT SURE WHAT IS GOING ON HERE
        //
        const G4VProcess* creator = aTrack->GetCreatorProcess();
        if (creator && creator->GetProcessName()=="OpWLS")
        {
            trajectory->WLS();
            trajectory->SetDrawTrajectory(true);
        }

        // Draw trajectories of those photons that hit SiPM
        if (trackInformation->GetTrackStatus() & hitSiPM) 
            trajectory->SetDrawTrajectory(true);
    }
    // Draw trajectories off all other particles
    else trajectory->SetDrawTrajectory(true);

    // Force drawing of trajectory
    if (trackInformation->GetForceDrawTrajectory()) trajectory->SetDrawTrajectory(true);

    if (fRecorder) fRecorder->RecordTrack(aTrack);
}
