
#include <unittest++/UnitTest++.h>
#include <unittest++/TestRunner.h>
#include <unittest++/TestReporterStdout.h>
#include <string.h>
#include <unordered_set>

#include "G4RunManager.hh"
#include "G4UImanager.hh"

#include "VKDetectorConstruction.hh"
#include "VKPhysicsList.hh"
#include "VKActionInitialization.hh"
#include "VKRecorderRoot.hh"

#include "G4HadronicProcessStore.hh"


using namespace UnitTest;


int main(int, const char *[])
{
    // get pointer to run manager
    G4RunManager *runManager = new G4RunManager;
    
    // recorder
    VKRecorderRoot *recorder = new VKRecorderRoot();

    // detector
    runManager->SetUserInitialization(new VKDetectorConstruction());
    // physics
    runManager->SetUserInitialization(new VKPhysicsList());
    // action
    runManager->SetUserInitialization(new VKActionInitialization(recorder));
    
    // get pointer to UI manager
    G4UImanager *UImanager = G4UImanager::GetUIpointer();

    // disable optical physics
    UImanager->ApplyCommand("/VK/physics/opticalPhysics false");
    
    // Only check for overlapping volumes when detector is initialized (above)
    UImanager->ApplyCommand("/VK/detector/volumes/checkOverlaps false");
    
    // Attempt to minimize verbosity
    UImanager->ApplyCommand("/process/em/verbose 0");
    G4HadronicProcessStore *hadProcStore = G4HadronicProcessStore::Instance();
    hadProcStore -> SetVerbose(0);
    
    // 
    // Info about models is being printed by G4EmModelManager::DumpModelList(verb).
    // This method is called from G4VEnergyLossProcess with verb=1.
    // Value can be changed by using G4VProcess::SetVerboseLevel(verb).
    // However, not clear to me how to do this in practice ...
    //
    
    // initialize run
    UImanager->ApplyCommand("/run/initialize");
    
//    // run all unit tests
//    return RunAllTests();

	// disable selected tests
    std::unordered_set<std::string> disabled = {
//			"VKEnergyDepositionTest",
//			"VKRecorderRootTest",
//			"VKBFieldTest",
//			"VKOpticalPhysicsTest",
//			"VKTransportationInBFieldTest",
//			"VKDetectorTest",
//			"VKPrimaryGeneratorTest"
    };
    TestReporterStdout reporter;
    TestRunner runner(reporter);
    return runner.RunTestsIf(Test::GetTestList(), nullptr,
                             [&](Test* t) {
                                 return !disabled.count(t->m_details.suiteName);
                             }, 0);
    
    // delete run manager
    delete G4RunManager::GetRunManager();
    
    return 0;
}
